@@ -38,6 +38,8 @@ from timelinecontrols import TimelineControls
 from pitivi.receiver import receiver, handler
 from zoominterface import Zoomable
 from pitivi.ui.common import LAYER_HEIGHT_EXPANDED, LAYER_SPACING
+from pitivi.timeline.timeline import MoveContext
+from pitivi.utils import Seeker
 
 # tooltip text for toolbar
 DELETE = _("Delete Selected")
@@ -130,9 +132,12 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self._factories = None
         self._finish_drag = False
         self._position = 0
+        self._state = gst.STATE_NULL
         self._createUI()
         self._prev_duration = 0
         self.shrink = True
+        self._seeker = Seeker(80)
+        self._seeker.connect('seek', self._seekerSeekCb)
 
     def _createUI(self):
         self.leftSizeGroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
@@ -141,8 +146,6 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
         # controls for tracks and layers
         self._controls = TimelineControls()
-        self._controls.connect('track-expanded',
-                self._timelineControlsTrackExpandedCb)
         controlwindow = gtk.ScrolledWindow(None, self.vadj)
         controlwindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
         controlwindow.add_with_viewport(self._controls)
@@ -230,10 +233,6 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self._canvas.connect("button-release-event", self._buttonRelease)
         self._canvas.connect("key-press-event", self._keyPressEventCb)
 
-    def _timelineControlsTrackExpandedCb(self, timeline_controls,
-            track, expanded):
-        self._canvas.setExpanded(track, expanded)
-
 
 ## Event callbacks
 
@@ -250,21 +249,30 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
             if kv == gtk.keysyms.Left:
                 if mod & gtk.gdk.SHIFT_MASK:
-                    self.project.pipeline.seekRelative(-gst.SECOND)
+                    self._seekRelative(-gst.SECOND)
                 elif mod & gtk.gdk.CONTROL_MASK:
-                    self.project.pipeline.seek(ltime+1)
+                    self._seeker.seek(ltime+1)
                 else:
-                    self.project.pipeline.seekRelative(-long(self.rate * gst.SECOND))
+                    self._seekRelative(-long(self.rate * gst.SECOND))
             elif kv == gtk.keysyms.Right:
                 if mod & gtk.gdk.SHIFT_MASK:
-                    self.project.pipeline.seekRelative(gst.SECOND)
+                    self._seekRelative(gst.SECOND)
                 elif mod & gtk.gdk.CONTROL_MASK:
-                    self.project.pipeline.seek(rtime+1)
+                    self._seeker.seek(rtime+1)
                 else:
-                    self.project.pipeline.seekRelative(long(self.rate * gst.SECOND))
+                    self._seekRelative(long(self.rate * gst.SECOND))
         finally:
             return True
 
+    def _seekRelative(self, time):
+        pipeline = self.project.pipeline
+        seekvalue = max(0, min(pipeline.getPosition() + time,
+            pipeline.getDuration()))
+        self._seeker.seek(seekvalue)
+
+    def _seekerSeekCb(self, seeker, position, format):
+        self.project.pipeline.seek(position, format)
+
     def _buttonPress(self, window, event):
         self.shrink = False
 
@@ -288,6 +296,9 @@ class Timeline(gtk.Table, Loggable, Zoomable):
             if not self._temp_objects:
                 self.timeline.disableUpdates()
                 self._add_temp_source()
+                focus = self._temp_objects[0]
+                self._move_context = MoveContext(self.timeline,
+                        focus, set(self._temp_objects[1:]))
             self._move_temp_source(x, y)
         return True
 
@@ -304,7 +315,11 @@ class Timeline(gtk.Table, Loggable, Zoomable):
     def _dragDropCb(self, widget, context, x, y, timestamp):
         self.app.action_log.begin("add clip")
         self._add_temp_source()
+        focus = self._temp_objects[0]
+        self._move_context = MoveContext(self.timeline,
+                focus, set(self._temp_objects[1:]))
         self._move_temp_source(x, y)
+        self._move_context.finish()
         self.app.action_log.commit()
         context.drop_finish(True, timestamp)
         self._factories = None
@@ -340,11 +355,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         x, y = self._canvas.convert_from_pixels(x - offset, y)
         priority = int((y // (LAYER_HEIGHT_EXPANDED + LAYER_SPACING)))
         delta = Zoomable.pixelToNs(x)
-        for obj in self._temp_objects:
-            obj.setStart(max(0, delta), snap=True)
-            obj.priority = priority
-            delta += obj.duration
-
+        self._move_context.editTo(delta, priority)
 
 ## Zooming and Scrolling
 
@@ -359,6 +370,9 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self._canvas._position = position
         self.scrollToPlayhead()
 
+    def stateChanged(self, state):
+        self._state = state
+
     def scrollToPlayhead(self):
         width = self.get_allocation().width
         new_pos = Zoomable.nsToPixel(self._position)
@@ -368,6 +382,8 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         return False
 
     def scrollToPosition(self, position):
+        if not self._state == gst.STATE_PLAYING:
+            return
         if position > self.hadj.upper:
             # we can't perform the scroll because the canvas needs to be
             # updated
